import requests

from jigu.error import TerraAPIError


class BaseAPI(object):
    def __init__(self, client):
        self._c = client

    def _api_get(self, path, unwrap=True, **kwargs):
        return self._handle_response(self._c.get(path, **kwargs), unwrap)

    def _api_post(self, path, unwrap=True, **kwargs):
        return self._handle_response(self._c.post(path, **kwargs), unwrap)

    def _handle_response(self, response: requests.Response, unwrap=True):
        try:
            res = response.json()
            if str(response.status_code).startswith("5"):
                if type(res) == dict:
                    raise TerraAPIError(res.get("error", None))
                else:
                    raise TerraAPIError(f"an error occured: {str(response.content)}")
            if str(response.status_code).startswith("2"):
                if unwrap:
                    return res["result"]
                else:
                    return res
        except ValueError:
            raise jigu.error.ClientError(f"invalid response {str(response.content)}")
