from typing import Union

import warnings

from jigu.api import BaseAPI
from jigu.core import AccAddress, Account, LazyGradedVestingAccount
from jigu.error import AccountNotFoundWarning


class AuthAPI(BaseAPI):
    def acc_info(self, address: AccAddress) -> Union[Account, LazyGradedVestingAccount]:
        info = self._api_get(f"/auth/accounts/{address}")
        if info["type"] == "core/Account":
            res = Account.deserialize(info)
        elif info["type"] == "core/LazyGradedVestingAccount":
            res = LazyGradedVestingAccount.deserialize(info)
        else:
            raise ValueError("could not deserialize account in auth.acc_info")
        if res.address is None:
            warnings.warn(
                "Account was not found; perhaps wrong chain or account needs to first be sent funds.",
                AccountNotFoundWarning,
            )
        return res
