import warnings
from typing import Any, Optional, Union

from jigu.api import BaseAPI
from jigu.core import Coin, Dec


class MarketAPI(BaseAPI):
    def swap_rate(self, offer_coin: Coin, ask_denom: str) -> Coin:
        if type(offer_coin.amount) != int:
            warnings.warn(
                f"Coin's amount will be truncated to integer: {int(offer_coin.amount)} {offer_coin.denom}",
                SyntaxWarning,
            )
        params = {
            "offer_coin": f"{int(offer_coin.amount)}{offer_coin.denom}",
            "ask_denom": ask_denom,
        }
        bid = self._api_get(f"/market/swap", params=params)
        return Coin.deserialize(bid)

    def terra_pool_delta(self) -> Dec:
        terra_pool_delta = self._api_get("/market/terra_pool_delta")
        return Dec.deserialize(terra_pool_delta)

    def params(self, key: Optional[str] = None) -> Union[dict, Any]:
        p = self._api_get("/market/parameters")
        p["pool_recovery_period"] = int(p["pool_recovery_period"])
        p["base_pool"] = Dec.deserialize(p["base_pool"])
        p["min_spread"] = Dec.deserialize(p["min_spread"])
        p["tobin_tax"] = Dec.deserialize(p["tobin_tax"])
        ill = p["illiquid_tobin_tax_list"]
        p["illiquid_tobin_tax_list"] = dict()
        for item in ill:
            p["illiquid_tobin_tax_list"][item["denom"]] = Dec.deserialize(
                item["tax_rate"]
            )
        if key is None:
            return p
        else:
            return p[key]
