from typing import List, Optional, Union

from jigu.api import BaseAPI
from jigu.core import (
    AccAddress,
    Coin,
    Coins,
    Dec,
    ExchangeRatePrevote,
    ExchangeRateVote,
    ValAddress,
)

from jigu.util.validation import validate_val_address


class OracleAPI(BaseAPI):

    # for some reason, votes and prevotes for a certain (val, denom) pair will give STILL give you a list
    def votes(
        self, validator: Optional[ValAddress] = None, denom: Optional[str] = None
    ):
        if validator is not None and denom is not None:
            validate_val_address(validator)
            votes = self._api_get(f"/oracle/denoms/{denom}/votes/{validator}")
        elif validator:
            validate_val_address(validator)
            votes = self._api_get(f"/oracle/voters/{validator}/votes")
        elif denom:
            votes = self._api_get(f"/oracle/denoms/{denom}/votes")
        else:
            raise ValueError("arguments validator and denom cannot both be None")
        return [ExchangeRateVote.deserialize(vote) for vote in votes]

    def prevotes(
        self, validator: Optional[ValAddress] = None, denom: Optional[str] = None
    ):
        if validator is not None and denom is not None:
            validate_val_address(validator)
            prevotes = self._api_get(f"/oracle/denoms/{denom}/prevotes/{validator}")
        elif validator:
            validate_val_address(validator)
            prevotes = self._api_get(f"/oracle/voters/{validator}/prevotes")
        elif denom:
            prevotes = self._api_get(f"/oracle/denoms/{denom}/prevotes")
        else:
            raise ValueError("arguments validator and denom cannot both be None")
        return [ExchangeRatePrevote.deserialize(prevote) for prevote in prevotes]

    def exchange_rates(self, denom: Optional[ValAddress] = None) -> Union[Coin, Coins]:
        res = self._api_get("/oracle/denoms/exchange_rates")
        rates = Coins.from_data(res)
        if denom is None:
            return rates
        else:
            return rates[denom]

    def active_denoms(self) -> List[str]:
        return self._api_get("/oracle/denoms/actives")

    def get_feeder_address(self, validator: ValAddress) -> AccAddress:
        validate_val_address(validator)
        return self._api_get(f"/oracle/voters/{validator}/feeder")

    def get_misses(self, validator: ValAddress) -> int:
        validate_val_address(validator)
        misses = self._api_get(f"/oracle/voters/{validator}/miss")
        return int(misses)

    def params(self, key: Optional[str] = None) -> Union[dict, int, Dec, List[str]]:
        p = self._api_get("/oracle/parameters")
        p["vote_period"] = int(p["vote_period"])
        p["vote_threshold"] = Dec(p["vote_threshold"])
        p["reward_band"] = Dec(p["reward_band"])
        p["reward_distribution_window"] = int(p["reward_distribution_window"])
        p["slash_fraction"] = Dec(p["slash_fraction"])
        p["slash_window"] = int(p["slash_window"])
        p["min_valid_per_window"] = Dec(p["min_valid_per_window"])
        if key is None:
            return p
        else:
            return p[key]
