from typing import Optional, Union

from jigu.api import BaseAPI
from jigu.core import Coin, Coins, Dec, PolicyConstraints
from jigu.core.denoms import uLuna


class TreasuryAPI(BaseAPI):
    def tax_cap(self, denom: str) -> Coin:
        cap = self._api_get(f"/treasury/tax_cap/{denom}")
        return Coin(denom, cap)

    def tax_rate(self) -> Dec:
        tax_rate = self._api_get("/treasury/tax_rate")
        return Dec.deserialize(tax_rate)

    def reward_weight(self) -> Dec:
        reward_weight = self._api_get("/treasury/reward_weight")
        return Dec.deserialize(reward_weight)

    def tax_proceeds(self, denom: Optional[str] = None) -> Coins:
        res = self._api_get("/treasury/tax_proceeds")
        tax_proceeds = Coins.deserialize(res)
        if denom is None:
            return tax_proceeds
        else:
            return tax_proceeds[denom]

    def seigniorage_proceeds(self) -> Coin:
        sp = self._api_get("/treasury/seigniorage_proceeds")
        return Coin(uLuna, int(sp))

    def params(self, key: Optional[str] = None) -> Union[dict, int, Dec]:
        p = self._api_get("/treasury/parameters")
        p["seigniorage_burden_target"] = Dec.deserialize(p["seigniorage_burden_target"])
        p["mining_increment"] = Dec.deserialize(p["mining_increment"])
        p["window_short"] = int(p["window_short"])
        p["window_long"] = int(p["window_long"])
        p["window_probation"] = int(p["window_probation"])
        p["tax_policy"] = PolicyConstraints.deserialize(p["tax_policy"])
        p["reward_policy"] = PolicyConstraints.deserialize(p["reward_policy"])
        if key is None:
            return p
        else:
            return p[key]
