from typing import Dict, List, Union

from jigu.api import BaseAPI
from jigu.core import Coin, SignedTx, StdTx, TxInfo, UnsignedTx, StdFee


class TxAPI(BaseAPI):
    def get_info(self, hash: str) -> TxInfo:
        tx_info = self._api_get(f"/txs/{hash}", unwrap=False)
        return TxInfo.deserialize(tx_info)

    def estimate_fee(
        self,
        tx: Union[UnsignedTx, SignedTx],
        gas_adjustment: Union[float, str],
        gas_prices: List[Coin],
    ) -> StdFee:
        data = {
            "tx": tx,
            "gas_adjustment": str(gas_adjustment),
            "gas_prices": gas_prices,
        }
        res = self._api_post("/txs/estimate_fee", data=data)
        gas = int(res["gas"])
        fees = [Coin.deserialize(coin) for coin in res["fees"]]
        return StdFee(gas=gas, amount=fees)

    def encode(self, tx: StdTx) -> str:
        res = self._api_post("/txs/encode", data=tx, unwrap=False)
        return res["tx"]

    def broadcast(self, tx: SignedTx, mode: str = "block"):
        if mode not in ["block", "sync", "async"]:
            raise ValueError(
                f"mode '{mode}' is not legal; mode can only be 'block', 'sync', or 'async'."
            )
        data = {"tx": tx.to_data()["value"], "mode": mode}
        res = self._api_post("/txs", data=data, unwrap=False)
        return res
