from __future__ import annotations

from jigu.core import Coins, AccAddress
from jigu.util.serdes import JSONDeserializable, JSONSerializable
from jigu.util.validation import validate_acc_address
from jigu.util.validation import Schemas as S


class MultiSendIO(JSONSerializable, JSONDeserializable):

    __schema__ = S.OBJECT(address=S.ACC_ADDRESS, coins=Coins.__schema__)

    def __init__(self, address: AccAddress, coins: Coins):
        validate_acc_address(address)
        self.address = address
        self.coins = Coins(coins)

    @classmethod
    def from_data(cls, data: dict) -> MultiSendIO:
        return cls(address=data["address"], coins=Coins.from_data(data["coins"]))


class Input(MultiSendIO):
    def __repr__(self) -> str:
        return f"Input(address='{self.address}', coins={self.coins!r})"


class Output(MultiSendIO):
    def __repr__(self) -> str:
        return f"Output(address='{self.address}', coins={self.coins!r})"
