from __future__ import annotations
from typing import Dict, Callable
from collections import defaultdict
import abc

from jigu.util import pretty_repr
from jigu.util.serdes import JSONDeserializable


class Event(dict, JSONDeserializable):
    def __init__(self, attributes: dict):
        dict.__init__(self, attributes)

    def __repr__(self) -> str:
        return pretty_repr("Event", ("Type:", self.type), *self.items())

    @property
    @abc.abstractmethod
    def type(self) -> str:
        raise NotImplementedError

    @classmethod
    @abc.abstractmethod
    def attribute_types(cls) -> Dict[str, Callable]:
        raise NotImplementedError

    @classmethod
    def from_data(cls, data: dict) -> Event:
        att_types = cls.attribute_types()
        atts = defaultdict(list)
        for att in data["attributes"]:
            key = att["key"]
            item = att_types[key](att["value"])  # convert each (k,v) pair
            atts[key].append(item)
        return cls(attributes=atts)


class MessageEvent(Event):

    type = "message"

    @classmethod
    def attribute_types(cls) -> Dict[str, Callable]:
        return {"module": str, "action": str, "sender": str}


EVENT_TYPES = {
    "message": MessageEvent,
}
