from __future__ import annotations
from typing import Dict, Any, Type
import abc

from jigu.util.serdes import JSONSerializable, JSONDeserializable
from jigu.util.validation import Schemas as S


class StdMsg(JSONSerializable, JSONDeserializable, metaclass=abc.ABCMeta):

    __schema__ = S.OBJECT(type=S.STRING, value={"type": "object"})

    @property
    @abc.abstractmethod
    def __type__(self):
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def __action__(self):
        raise NotImplementedError

    def msg_value(self) -> Dict[str, Any]:
        return dict(self.__dict__)

    def to_data(self) -> Dict[str, Any]:
        return {"type": self.__type__, "value": self.msg_value()}


from .bank import MsgSend, MsgMultiSend
from .distribution import (
    MsgModifyWithdrawAddress,
    MsgWithdrawDelegationReward,
    MsgWithdrawValidatorCommission,
)
from .oracle import MsgExchangeRatePrevote, MsgExchangeRateVote, MsgDelegateFeedConsent
from .gov import MsgSubmitProposal, MsgVote, MsgDeposit
from .market import MsgSwap
from .staking import (
    MsgBeginRedelegate,
    MsgDelegate,
    MsgUndelegate,
    MsgEditValidator,
    MsgCreateValidator,
)
from .slashing import MsgUnjail

MSG_TYPES = {
    "bank/MsgSend": MsgSend,
    "bank/MsgMultiSend": MsgMultiSend,
    "distribution/MsgModifyWithdrawAddress": MsgModifyWithdrawAddress,
    "distribution/MsgWithdrawDelegationReward": MsgWithdrawDelegationReward,
    "distribution/MsgWithdrawValidatorCommission": MsgWithdrawValidatorCommission,
    "oracle/MsgExchangeRatePrevote": MsgExchangeRatePrevote,
    "oracle/MsgExchangeRateVote": MsgExchangeRateVote,
    "oracle/MsgDelegateFeedConsent": MsgDelegateFeedConsent,
    "gov/MsgSubmitProposal": MsgSubmitProposal,
    "gov/MsgDeposit": MsgDeposit,
    "gov/MsgVote": MsgVote,
    "market/MsgSwap": MsgSwap,
    "staking/MsgBeginRedelegate": MsgBeginRedelegate,
    "staking/MsgDelegate": MsgDelegate,
    "staking/MsgUndelegate": MsgUndelegate,
    "staking/MsgEditValidator": MsgEditValidator,
    "staking/MsgCreateValidator": MsgCreateValidator,
    "cosmos/MsgUnjail": MsgUnjail,
}
