from __future__ import annotations
from typing import Any, Dict, Union
from dataclasses import dataclass

from jigu.core import Coin, Dec
from jigu.util import pretty_repr
from jigu.util.serdes import JSONDeserializable, JSONSerializable
from jigu.util.validation import Schemas as S


@dataclass
class PolicyConstraints(JSONSerializable, JSONDeserializable):

    __schema__ = S.OBJECT(
        rate_min=Dec.__schema__,
        rate_max=Dec.__schema__,
        cap=Coin.__schema__,
        change_max=Dec.__schema__,
    )

    rate_min: Dec
    rate_max: Dec
    cap: Coin
    change_max: Dec

    @classmethod
    def from_data(cls, data: Dict[str, Any]) -> PolicyConstraints:
        return cls(
            rate_min=Dec(data["rate_min"]),
            rate_max=Dec(data["rate_max"]),
            cap=Coin.from_data(data["cap"]),
            change_max=Dec(data["change_max"]),
        )
