class ValidationError(Exception):
    """Ambiguous validation check failed."""


class DenomIncompatibleError(ValueError):
    """An operation requiring compatible Coins encountered incompatible denoms."""


class InvalidAccAddress(ValueError):
    """An incorrect value was passed into a method that expects a valid Account
    address (Bech32-encoded address prefixed with terra-)."""


class InvalidValAddress(ValueError):
    """An incorrect value was passed into a method that expects a valid Validator
    (operator) address (Bech32-encoded pubkey prefixed with terrapub-)."""


class InvalidAccPubKey(ValueError):
    """An incorrect value was passed into a method that expects a valid Account
    public key (Bech32-encoded pubkey prefixed with terrapub-)."""


class InvalidValPubKey(ValueError):
    """An incorrect value was passed into a method that expects a valid Validator
    public key (Bech32-encoded pubkey prefixed with terravalpub-)."""


class InvalidValConsPubKey(ValueError):
    """An incorrect value was passed into a method that expects a valid Validator
    Consensus (Tendermint) pubkey (Bech32-encoded pubkey prefixed with terravalconspub-)."""


# Runtime exceptions


class ClientError(Exception):
    pass


class TerraAPIError(Exception):
    pass


class DenomNotFoundError(KeyError):
    pass


# Warnings


class AccountNotFoundWarning(Warning):
    pass
