from __future__ import annotations
from bech32 import bech32_encode, convertbits
import uuid
import hashlib
import base64

from typing import Any, Optional, List, Dict, Callable, Tuple


def get_bech(prefix: str, payload: str) -> str:
    return bech32_encode(
        prefix, convertbits(bytes.fromhex(payload), 8, 5)
    )  # base64 -> base32


def generate_salt() -> str:
    """Generate a 4 bytes salt."""
    return uuid.uuid4().hex[:4]


def hash_amino(txdata: str) -> str:
    """Get the transaction hash from Amino-encoded Transaction in base64."""
    return hashlib.sha256(base64.b64decode(txdata)).digest().hex()


def list_to_dict(key: str, deserialize: Callable) -> Callable:
    def transform_list_to_dict(data: Optional[List[Dict[str, Any]]]) -> Dict[str, Any]:
        if data:
            return {o[key]: deserialize(o) for o in data}
        return {}

    return transform_list_to_dict


def pretty_repr(title: str, *items: Tuple[str, str]) -> str:
    """Returns a basic fancy __repr__"""
    newline = "\n"
    return f"""--{title}--
{newline.join(f"  {item[0]:<25}{item[1]}" for item in items)}
"""
